/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.model;

import java.net.URLEncoder;

import com.redhat.rhevm.api.common.util.StatusUtils;
import com.redhat.rhevm.api.common.util.StringUtils;
import com.redhat.rhevm.api.model.Creation;
import com.redhat.rhevm.api.model.Fault;
import com.redhat.rhevm.api.model.CreationStatus;
import com.redhat.rhevm.api.powershell.enums.PowerShellAsyncTaskResult;
import com.redhat.rhevm.api.powershell.enums.PowerShellAsyncTaskStatus;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;

public class PowerShellAsyncTask {

    private static final String TASK_TYPE = "RhevmCmd.CLITask";
    private static final String STATUS_TYPE = "VdcCommon.AsyncTasks.AsyncTaskStatus";
    private static final String ID_SEPARATOR = ",";
    private static final String FAILURE_REASON = "RHEVM asynchronous task failed";

    public static boolean isTask(PowerShellParser.Entity entity) {
        return TASK_TYPE.equals(entity.getType());
    }

    public static boolean isStatus(PowerShellParser.Entity entity) {
        return STATUS_TYPE.equals(entity.getType());
    }

    public static String parseTask(PowerShellParser.Entity entity, String taskIds) {
        String id = "";
        if (isTask(entity)) {
            id = entity.get("taskid");
        }
        return taskIds != null ? taskIds + URLEncoder.encode(ID_SEPARATOR) + id : id;
    }

    public static String[] parseTaskIds(String taskIds) {
        return taskIds.split(ID_SEPARATOR);
    }

    public static CreationStatus parseStatus(PowerShellParser.Entity entity, CreationStatus cumlative) {
        CreationStatus ret = cumlative;
        if (isStatus(entity)) {
            Boolean statusPresent =
                entity.isSet("statusspecified")
                ? entity.get("statusspecified", Boolean.class)
                : Boolean.TRUE;
            PowerShellAsyncTaskStatus status =
                statusPresent
                ? entity.get("status", PowerShellAsyncTaskStatus.class)
                : PowerShellAsyncTaskStatus.unknown;
            Boolean resultPresent =
                entity.isSet("resultspecified")
                ? entity.get("resultspecified", Boolean.class)
                : Boolean.TRUE;
            PowerShellAsyncTaskResult result =
                resultPresent
                ? entity.get("result", PowerShellAsyncTaskResult.class)
                : null;

            switch(status) {
                case unknown:
                case init:
                    ret = cumlative != CreationStatus.FAILED ? CreationStatus.PENDING : cumlative;
                    break;
                case running:
                    ret = cumlative != CreationStatus.FAILED ? CreationStatus.IN_PROGRESS : cumlative;
                    break;
                case finished:
                    if (result == PowerShellAsyncTaskResult.success) {
                        ret = cumlative != null ? cumlative : CreationStatus.COMPLETE;
                    } else {
                        ret = CreationStatus.FAILED;
                    }
                    break;
                case cleaning:
                    if (result == PowerShellAsyncTaskResult.cleanSuccess) {
                        ret = cumlative != null ? cumlative : CreationStatus.COMPLETE;
                    } else {
                        ret = CreationStatus.FAILED;
                    }
                    break;
                case aborting:
                    ret = CreationStatus.FAILED;
                    break;
            }
        }
        return ret;
    }

    public static Creation parse(PowerShellParser parser, String output) {
        Creation creation = new Creation();

        for (PowerShellParser.Entity entity : parser.parse(output)) {
            if (isStatus(entity)) {
                String status = creation.getStatus()!=null ? creation.getStatus().getState() : null;
                creation.setStatus(StatusUtils.create(parseStatus(entity, StringUtils.isNullOrEmpty(status) ? CreationStatus.FAILED : CreationStatus.fromValue(status))));
                if (CreationStatus.FAILED.value().equals(status)) {
                    creation.setFault(new Fault());
                    creation.getFault().setReason(FAILURE_REASON);
                    creation.getFault().setDetail(entity.isSet("exception") ? entity.get("exception") : entity.get("message"));
                    break;
                }
            }
        }

        return creation;
    }

}
